Package: tune
Title: Tidy Tuning Tools
Version: 2.1.0
Authors@R: c(
    person("Max", "Kuhn", , "max@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-2402-136X")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: The ability to tune models is important. 'tune' contains
    functions and classes to be used in conjunction with other
    'tidymodels' packages for finding reasonable values of
    hyper-parameters in models, preprocessing methods, and post-processing
    steps.
License: MIT + file LICENSE
URL: https://tune.tidymodels.org/, https://github.com/tidymodels/tune
BugReports: https://github.com/tidymodels/tune/issues
Depends: R (>= 4.1)
Imports: cli (>= 3.3.0), dials (>= 1.4.3), dplyr (>= 1.1.0), GauPro (>=
        0.2.15), generics (>= 0.1.2), ggplot2, glue (>= 1.6.2), hardhat
        (>= 1.4.2), lifecycle, parallel, parsnip (>= 1.5.0), purrr (>=
        1.0.0), recipes (>= 1.3.2), rlang (>= 1.1.4), rsample (>=
        1.3.2), tailor (>= 0.1.0), tibble (>= 3.1.0), tidyr (>= 1.2.0),
        tidyselect (>= 1.1.2), vctrs (>= 0.6.1), withr, workflows (>=
        1.3.0), yardstick (>= 1.4.0)
Suggests: C50, censored (>= 0.3.0), covr, future (>= 1.33.0),
        future.apply, kernlab, kknn, knitr, mgcv, mirai (>= 2.4.0),
        modeldata, probably, scales, spelling, splines2, survival,
        testthat (>= 3.0.0), xgboost, xml2
Config/Needs/website: pkgdown, tidymodels, kknn, tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-17 04:15:55 UTC; max
Author: Max Kuhn [aut, cre] (ORCID: <https://orcid.org/0000-0003-2402-136X>),
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Max Kuhn <max@posit.co>
Repository: CRAN
Date/Publication: 2026-04-17 14:30:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:33:06 UTC; unix
