# Generated by roxygen2: do not edit by hand

S3method("[",iteration_results)
S3method("[",resample_results)
S3method("[",tune_results)
S3method("names<-",iteration_results)
S3method("names<-",resample_results)
S3method("names<-",tune_results)
S3method(.get_fingerprint,tune_results)
S3method(.use_case_weights_with_yardstick,default)
S3method(.use_case_weights_with_yardstick,hardhat_frequency_weights)
S3method(.use_case_weights_with_yardstick,hardhat_importance_weights)
S3method(augment,last_fit)
S3method(augment,resample_results)
S3method(augment,tune_results)
S3method(autoplot,resample_results)
S3method(autoplot,tune_results)
S3method(collect_extracts,default)
S3method(collect_extracts,tune_results)
S3method(collect_metrics,default)
S3method(collect_metrics,tune_results)
S3method(collect_notes,default)
S3method(collect_notes,tune_results)
S3method(collect_predictions,default)
S3method(collect_predictions,tune_results)
S3method(compute_metrics,default)
S3method(compute_metrics,tune_results)
S3method(dplyr_reconstruct,iteration_results)
S3method(dplyr_reconstruct,resample_results)
S3method(dplyr_reconstruct,tune_results)
S3method(extract_fit_engine,tune_results)
S3method(extract_fit_parsnip,tune_results)
S3method(extract_mold,tune_results)
S3method(extract_preprocessor,tune_results)
S3method(extract_recipe,tune_results)
S3method(extract_spec_parsnip,tune_results)
S3method(extract_workflow,last_fit)
S3method(extract_workflow,tune_results)
S3method(fit_best,default)
S3method(fit_best,tune_results)
S3method(fit_resamples,default)
S3method(fit_resamples,model_spec)
S3method(fit_resamples,workflow)
S3method(int_pctl,tune_results)
S3method(last_fit,default)
S3method(last_fit,model_fit)
S3method(last_fit,model_spec)
S3method(last_fit,workflow)
S3method(load_pkgs,character)
S3method(load_pkgs,model_spec)
S3method(load_pkgs,workflow)
S3method(merge,model_spec)
S3method(merge,recipe)
S3method(min_grid,C5_rules)
S3method(min_grid,boost_tree)
S3method(min_grid,cubist_rules)
S3method(min_grid,linear_reg)
S3method(min_grid,logistic_reg)
S3method(min_grid,mars)
S3method(min_grid,model_spec)
S3method(min_grid,multinom_reg)
S3method(min_grid,nearest_neighbor)
S3method(min_grid,pls)
S3method(min_grid,poisson_reg)
S3method(min_grid,proportional_hazards)
S3method(min_grid,rule_fit)
S3method(obj_sum,resample_results)
S3method(obj_sum,tune_race)
S3method(obj_sum,tune_results)
S3method(outcome_names,formula)
S3method(outcome_names,recipe)
S3method(outcome_names,terms)
S3method(outcome_names,tune_results)
S3method(outcome_names,workflow)
S3method(outcome_names,workflow_variables)
S3method(predict,conf_bound)
S3method(predict,exp_improve)
S3method(predict,last_fit)
S3method(predict,prob_improve)
S3method(predict,tune_results)
S3method(pretty,tune_results)
S3method(print,control_bayes)
S3method(print,control_grid)
S3method(print,control_last_fit)
S3method(print,prob_improve)
S3method(print,tune_results)
S3method(select_best,default)
S3method(select_best,tune_results)
S3method(select_by_one_std_err,default)
S3method(select_by_one_std_err,tune_results)
S3method(select_by_pct_loss,default)
S3method(select_by_pct_loss,tune_results)
S3method(show_best,default)
S3method(show_best,tune_results)
S3method(size_sum,tune_results)
S3method(tune_bayes,default)
S3method(tune_bayes,model_spec)
S3method(tune_bayes,workflow)
S3method(tune_grid,default)
S3method(tune_grid,model_spec)
S3method(tune_grid,workflow)
S3method(vec_cast,data.frame.iteration_results)
S3method(vec_cast,data.frame.resample_results)
S3method(vec_cast,data.frame.tune_results)
S3method(vec_cast,iteration_results.data.frame)
S3method(vec_cast,iteration_results.iteration_results)
S3method(vec_cast,iteration_results.tbl_df)
S3method(vec_cast,resample_results.data.frame)
S3method(vec_cast,resample_results.resample_results)
S3method(vec_cast,resample_results.tbl_df)
S3method(vec_cast,tbl_df.iteration_results)
S3method(vec_cast,tbl_df.resample_results)
S3method(vec_cast,tbl_df.tune_results)
S3method(vec_cast,tune_results.data.frame)
S3method(vec_cast,tune_results.tbl_df)
S3method(vec_cast,tune_results.tune_results)
S3method(vec_ptype2,data.frame.iteration_results)
S3method(vec_ptype2,data.frame.resample_results)
S3method(vec_ptype2,data.frame.tune_results)
S3method(vec_ptype2,iteration_results.data.frame)
S3method(vec_ptype2,iteration_results.iteration_results)
S3method(vec_ptype2,iteration_results.tbl_df)
S3method(vec_ptype2,resample_results.data.frame)
S3method(vec_ptype2,resample_results.resample_results)
S3method(vec_ptype2,resample_results.tbl_df)
S3method(vec_ptype2,tbl_df.iteration_results)
S3method(vec_ptype2,tbl_df.resample_results)
S3method(vec_ptype2,tbl_df.tune_results)
S3method(vec_ptype2,tune_results.data.frame)
S3method(vec_ptype2,tune_results.tbl_df)
S3method(vec_ptype2,tune_results.tune_results)
S3method(vec_restore,iteration_results)
S3method(vec_restore,resample_results)
S3method(vec_restore,tune_results)
export(.catch_and_log)
export(.check_grid)
export(.check_param_objects)
export(.config_key_from_metrics)
export(.create_weight_mapping)
export(.determine_pred_types)
export(.effective_sample_size)
export(.estimate_metrics)
export(.filter_perf_metrics)
export(.get_config_key)
export(.get_data_subsets)
export(.get_extra_col_names)
export(.get_fingerprint)
export(.get_resample_weights)
export(.get_tune_eval_time_target)
export(.get_tune_eval_times)
export(.get_tune_metric_names)
export(.get_tune_metrics)
export(.get_tune_outcome_names)
export(.get_tune_parameter_names)
export(.get_tune_parameters)
export(.get_tune_workflow)
export(.has_preprocessor)
export(.has_preprocessor_formula)
export(.has_preprocessor_recipe)
export(.has_preprocessor_variables)
export(.has_spec)
export(.is_cataclysmic)
export(.load_namespace)
export(.loop_over_all_stages)
export(.loop_over_all_stages2)
export(.make_static)
export(.needs_finalization)
export(.par_fns)
export(.set_workflow)
export(.set_workflow_recipe)
export(.set_workflow_spec)
export(.stash_last_result)
export(.update_model)
export(.update_parallel_over)
export(.update_recipe)
export(.use_case_weights_with_yardstick)
export(.validate_resample_weights)
export(.weighted_sd)
export(add_resample_weights)
export(augment)
export(autoplot)
export(calculate_resample_weights)
export(check_eval_time_arg)
export(check_initial)
export(check_metric_in_tune_results)
export(check_metrics)
export(check_metrics_arg)
export(check_parameters)
export(check_rset)
export(check_time)
export(check_workflow)
export(choose_eval_time)
export(choose_framework)
export(choose_metric)
export(collect_extracts)
export(collect_metrics)
export(collect_notes)
export(collect_predictions)
export(compute_metrics)
export(conf_bound)
export(conf_mat_resampled)
export(control_bayes)
export(control_grid)
export(control_last_fit)
export(control_resamples)
export(coord_obs_pred)
export(empty_ellipses)
export(encode_set)
export(estimate_tune_results)
export(eval_mirai)
export(exp_improve)
export(expo_decay)
export(extract_fit_engine)
export(extract_fit_parsnip)
export(extract_mold)
export(extract_parameter_set_dials)
export(extract_preprocessor)
export(extract_recipe)
export(extract_resample_weights)
export(extract_spec_parsnip)
export(extract_workflow)
export(filter_parameters)
export(finalize_model)
export(finalize_recipe)
export(finalize_tailor)
export(finalize_workflow)
export(finalize_workflow_preprocessor)
export(first_eval_time)
export(first_metric)
export(fit_best)
export(fit_max_value)
export(fit_resamples)
export(forge_from_workflow)
export(future_installed)
export(get_future_workers)
export(get_metric_time)
export(get_mirai_workers)
export(get_parallel_seeds)
export(get_tune_colors)
export(has_non_par_pkgs)
export(initialize_catalog)
export(int_pctl)
export(is_preprocessor)
export(is_recipe)
export(is_workflow)
export(last_fit)
export(load_pkgs)
export(loop_call)
export(maybe_choose_eval_time)
export(message_wrap)
export(metrics_info)
export(min_grid)
export(min_grid.C5_rules)
export(min_grid.boost_tree)
export(min_grid.cubist_rules)
export(min_grid.linear_reg)
export(min_grid.logistic_reg)
export(min_grid.mars)
export(min_grid.multinom_reg)
export(min_grid.nearest_neighbor)
export(min_grid.pls)
export(min_grid.poisson_reg)
export(min_grid.proportional_hazards)
export(min_grid.rule_fit)
export(mirai_installed)
export(new_backend_options)
export(new_bare_tibble)
export(new_iteration_results)
export(outcome_names)
export(parameters)
export(prob_improve)
export(pull_rset_attributes)
export(required_pkgs)
export(schedule_grid)
export(select_best)
export(select_by_one_std_err)
export(select_by_pct_loss)
export(show_best)
export(show_notes)
export(tunable)
export(tune)
export(tune_args)
export(tune_bayes)
export(tune_grid)
export(val_class_and_single)
export(val_class_or_null)
import(rlang)
import(vctrs)
import(workflows)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_alert_warning)
importFrom(cli,cli_inform)
importFrom(cli,cli_warn)
importFrom(cli,qty)
importFrom(dials,encode_unit)
importFrom(dials,is_unknown)
importFrom(dials,parameters)
importFrom(dplyr,all_of)
importFrom(dplyr,anti_join)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,dplyr_reconstruct)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,inner_join)
importFrom(dplyr,last)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_if)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,sample_n)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,starts_with)
importFrom(dplyr,ungroup)
importFrom(dplyr,vars)
importFrom(generics,augment)
importFrom(generics,min_grid)
importFrom(generics,required_pkgs)
importFrom(generics,tunable)
importFrom(generics,tune_args)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(glue,glue)
importFrom(glue,glue_collapse)
importFrom(hardhat,extract_fit_engine)
importFrom(hardhat,extract_fit_parsnip)
importFrom(hardhat,extract_mold)
importFrom(hardhat,extract_parameter_set_dials)
importFrom(hardhat,extract_preprocessor)
importFrom(hardhat,extract_recipe)
importFrom(hardhat,extract_spec_parsnip)
importFrom(hardhat,extract_workflow)
importFrom(hardhat,tune)
importFrom(lifecycle,deprecated)
importFrom(parsnip,get_from_env)
importFrom(parsnip,required_pkgs)
importFrom(purrr,map_int)
importFrom(recipes,all_outcomes)
importFrom(recipes,all_predictors)
importFrom(rlang,"%||%")
importFrom(rlang,":=")
importFrom(rlang,call2)
importFrom(rlang,call_name)
importFrom(rlang,caller_env)
importFrom(rlang,env_get)
importFrom(rlang,eval_tidy)
importFrom(rlang,expr)
importFrom(rlang,is_false)
importFrom(rlang,is_function)
importFrom(rlang,is_missing)
importFrom(rlang,is_quosure)
importFrom(rlang,is_quosures)
importFrom(rlang,ns_env)
importFrom(rlang,quo_get_expr)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(rsample,.get_fingerprint)
importFrom(rsample,int_pctl)
importFrom(stats,dnorm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(tibble,obj_sum)
importFrom(tibble,size_sum)
importFrom(tidyr,nest)
importFrom(tidyr,unnest)
importFrom(yardstick,accuracy)
importFrom(yardstick,brier_class)
importFrom(yardstick,brier_survival)
importFrom(yardstick,rmse)
importFrom(yardstick,roc_auc)
importFrom(yardstick,rsq)
importFrom(yardstick,weighted_interval_score)
