# percentile intervals - resamples only

    Code
      int_res_1 <- int_pctl(lm_res, times = 500)
    Condition
      Warning in `rsample::int_pctl()`:
      Recommend at least 1000 non-missing bootstrap resamples for terms `rmse` and `rsq`.

---

    Code
      int_pctl(lm_res, times = 2000, metrics = "rmse")
    Condition
      Error in `int_pctl()`:
      ! `metrics` should be a metric set as generated by `yardstick::metric_set()`.

---

    Code
      int_res_2 <- int_pctl(lm_res, times = 500, alpha = 0.25)
    Condition
      Warning in `rsample::int_pctl()`:
      Recommend at least 1000 non-missing bootstrap resamples for terms `rmse` and `rsq`.

# percentile intervals - last fit

    Code
      int_res_1 <- int_pctl(lm_res, times = 200)
    Condition
      Warning in `rsample::int_pctl()`:
      Recommend at least 1000 non-missing bootstrap resamples for term `mae`.

---

    Code
      int_res_2 <- int_pctl(lm_res, times = 200)
    Condition
      Warning in `rsample::int_pctl()`:
      Recommend at least 1000 non-missing bootstrap resamples for term `mae`.

# percentile intervals - grid + bayes tuning

    Code
      int_res_1 <- int_pctl(c5_res)

---

    Code
      set.seed(92)
      c5_bo_res <- tune_bayes(set_mode(set_engine(decision_tree(min_n = tune()),
      "C5.0"), "classification"), Class ~ ., resamples = cls_rs, initial = c5_res,
      iter = 1, metrics = metric_set(sens), control = control_bayes(save_pred = TRUE))

# percentile intervals - grid tuning with validation set

    Code
      int_res_1 <- int_pctl(c5_res)

