# vec_cast() is working

    Code
      vec_cast(x, x)
    Condition
      Error in `stop_incompatible_cast_resample_results()`:
      ! Can't convert `x` <resample_results> to <resample_results>.
      Can't cast to a <resample_results> because attributes are likely incompatible.

---

    Code
      vec_cast(tbl, x)
    Condition
      Error in `stop_incompatible_cast_resample_results()`:
      ! Can't convert `tbl` <tibble> to <resample_results>.
      Can't cast to a <resample_results> because attributes are likely incompatible.

---

    Code
      vec_cast(df, x)
    Condition
      Error in `stop_incompatible_cast_resample_results()`:
      ! Can't convert `df` <data.frame> to <resample_results>.
      Can't cast to a <resample_results> because attributes are likely incompatible.

---

    Code
      vec_cast(x, x)
    Condition
      Error in `stop_incompatible_cast_tune_results()`:
      ! Can't convert `x` <tune_results> to <tune_results>.
      Can't cast to a <tune_results> because attributes are likely incompatible.

---

    Code
      vec_cast(tbl, x)
    Condition
      Error in `stop_incompatible_cast_tune_results()`:
      ! Can't convert `tbl` <tibble> to <tune_results>.
      Can't cast to a <tune_results> because attributes are likely incompatible.

---

    Code
      vec_cast(df, x)
    Condition
      Error in `stop_incompatible_cast_tune_results()`:
      ! Can't convert `df` <data.frame> to <tune_results>.
      Can't cast to a <tune_results> because attributes are likely incompatible.

---

    Code
      vec_cast(x, x)
    Condition
      Error in `stop_incompatible_cast_iteration_results()`:
      ! Can't convert `x` <iteration_results> to <iteration_results>.
      Can't cast to a <iteration_results> because attributes are likely incompatible.

---

    Code
      vec_cast(tbl, x)
    Condition
      Error in `stop_incompatible_cast_iteration_results()`:
      ! Can't convert `tbl` <tibble> to <iteration_results>.
      Can't cast to a <iteration_results> because attributes are likely incompatible.

---

    Code
      vec_cast(df, x)
    Condition
      Error in `stop_incompatible_cast_iteration_results()`:
      ! Can't convert `df` <data.frame> to <iteration_results>.
      Can't cast to a <iteration_results> because attributes are likely incompatible.

