% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checks.R
\name{check_resample_weights}
\alias{check_resample_weights}
\title{Check fold weights in rset objects}
\usage{
check_resample_weights(x)
}
\arguments{
\item{x}{An rset object.}
}
\value{
\code{NULL} invisibly, or error if weights are invalid.
}
\description{
Check fold weights in rset objects
}
\keyword{internal}
